@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#!/usr/bin/env perl
 #
 # Modify the config.json.sample file, save it to config.json. Run server.pl on
 # server, run client.pl on localhost. Config your browser to use socks 5 proxy
@@ -8,7 +8,6 @@
 use 5.006;
 use strict;
 use warnings;
-use Getopt::Long;
 use JSON;
 use Net::Shadowsocks::Server;
 
