@@ -41,7 +41,7 @@ use Digest::MD5;
 #use Mcrypt qw(:FUNCS);
 #use String::HexConvert ':all';
 
-our $VERSION = '0.9.3';
+our $VERSION = '0.9.3.1';
 
 our %_ciphers = 
     (
@@ -160,7 +160,8 @@ sub _initialize_cipher($$)
                 $_encryptor = Crypt::NaCl::Sodium->aead();
                 #$_key = $_encryptor->chacha20_keygen();
                 $_key = _EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
-                my $_encrypt_salt = makerandom_octet( Length => $_ciphers{$_method}->[2]); 
+                #my $_encrypt_salt = makerandom_octet( Length => $_ciphers{$_method}->[2]); 
+                my $_encrypt_salt = random_bytes( $_ciphers{$_method}->[2]); 
                 $_encrypt_subkey = hkdf($_key,$_encrypt_salt,'SHA1',$_ciphers{$_method}->[2],"ss-subkey");
                 #carp ascii_to_hex($_encrypt_subkey);
                 $_decryptor = Crypt::NaCl::Sodium->aead();
@@ -174,7 +175,8 @@ sub _initialize_cipher($$)
     elsif($_method =~ /gcm$/ )
      {
             $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
-            my $_encrypt_salt = makerandom_octet( Length => $_ciphers{$_method}->[2]); 
+            #my $_encrypt_salt = makerandom_octet( Length => $_ciphers{$_method}->[2]); 
+            my $_encrypt_salt = random_bytes( $_ciphers{$_method}->[2]); 
             my $_encrypt_subkey = hkdf($_key,$_encrypt_salt,'SHA1',$_ciphers{$_method}->[2],"ss-subkey");
             $_iv = $_encrypt_salt; 
             $_encryptor = Crypt::AuthEnc::GCM->new('AES', $_encrypt_subkey);
@@ -243,7 +245,7 @@ sub _initialize_cipher($$)
 
         $_key = _EVP_BytesToKey(16, 16, '', $_password, 1 );
 #        $_iv = makerandom_octet( Length => 16 );
-        $_iv = random_bytes($_ciphers{$_method}->[3]);
+        $_iv = random_bytes(16);
         $_encryptor = Crypt::Spritz::Cipher->new($_key, $_iv);
     }
     
@@ -353,7 +355,7 @@ Net::Shadowsocks - the asynchronous, non-blocking shadowsocks client and server.
 
 =head1 VERSION
 
-Version 0.9.3
+Version 0.9.3.1
 
 =head1 SYNOPSIS
     
