@@ -32,7 +32,7 @@ package Net::Shadowsocks::Client;
     use Socket qw(IPPROTO_TCP MSG_FASTOPEN SOL_SOCKET SO_REUSEPORT SO_REUSEADDR);
     #use String::HexConvert ':all';
     
-    our $VERSION = '0.9.3';
+    our $VERSION = '0.9.3.1';
     $AnyEvent::Log::FILTER->level ("info");
     sub new($$$$$$) 
     {
@@ -208,7 +208,8 @@ package Net::Shadowsocks::Client;
                                             {
                                                 my $rhandle = shift;
                                                 my $incoming_data = $rhandle->rbuf();
-                                                my $decrypted_data = undef;
+                                                #my $decrypted_data = undef;
+                                                my $decrypted_data = "";
                                                 unless ( defined($server_iv) ) 
                                                 {
                                                     if (length($incoming_data) < length($iv))
@@ -285,9 +286,10 @@ package Net::Shadowsocks::Client;
                                                         }
                                                         else
                                                         {
-                                                         #my $chunk_len = 0;
+                                                            #my $data_len_total = 0;
                                                             while (length($incoming_data) > 0)
                                                             {
+                                                                #carp length($incoming_data) . " bytes received";
                                                                 if (length($incoming_data) < 34)
                                                                 {
                                                                     return;
@@ -298,7 +300,7 @@ package Net::Shadowsocks::Client;
                                                                     my $data_len_pt;
                                                                     eval
                                                                     {
-                                                                        $data_len_pt = $decryptor->ietf_decrypt($data_len_ct, '', $decrypt_nonce, $decrypt_subkey);
+                                                                        $data_len_pt = $decryptor->ietf_decrypt($data_len_ct, "", $decrypt_nonce, $decrypt_subkey);
                                                                     };
                                                                     if ( $@ ) 
                                                                     {
@@ -307,11 +309,12 @@ package Net::Shadowsocks::Client;
                                                                     } 
                                                                     else 
                                                                     {
+                                                                        #$decrypt_nonce = $decrypt_nonce->increment();
                                                                         my $data_len = unpack('n',$data_len_pt); 
-                                                                        #carp "Decrypted data length: $data_len\n";
+                                                                        #carp "Decrypted data length: $data_len";
                                                                         my $chunk_len = $data_len + 34;
                                                                     
-                                                                        #$chunk_len_total += $chunk_len;
+                                                                        #$data_len_total += $data_len;
                                                                         if (length($incoming_data) < $chunk_len)
                                                                         {
                                                                             return;
@@ -319,7 +322,7 @@ package Net::Shadowsocks::Client;
                                                                         else
                                                                         {
                                                                             $decrypt_nonce = $decrypt_nonce->increment();
-                                                                            $decrypted_data .= $decryptor->ietf_decrypt(substr($incoming_data,18,$data_len + 16),'',$decrypt_nonce,$decrypt_subkey);
+                                                                            $decrypted_data .= $decryptor->ietf_decrypt(substr($incoming_data,18,$data_len + 16),"",$decrypt_nonce,$decrypt_subkey);
                                                                             if ( $@ ) 
                                                                             {
                                                                                 AE::log error =>  "data forged!";
@@ -330,12 +333,14 @@ package Net::Shadowsocks::Client;
                                                                                 $decrypt_nonce = $decrypt_nonce->increment(); 
                                                                                 $incoming_data = substr($incoming_data,$chunk_len);                                                                       
                                                                                 $rhandle->{rbuf} = $incoming_data;
-                                                                                carp "more chunk to process" if (length($incoming_data ) > 0) 
+                                                                                #carp length($incoming_data);
                                                                             }  
                                                                         }      
                                                                     }
                                                                 }
                                                             }
+                                                            #carp $data_len_total;
+                                                            #carp length($decrypted_data);
                                                         }
                                                     }
                                                     elsif($self->{method} =~ /gcm$/)
@@ -352,7 +357,7 @@ package Net::Shadowsocks::Client;
                                                                 my $data_len_ct = substr($incoming_data,0,2); 
                                                                 $decryptor->reset();
                                                                 $decryptor->iv_add($decrypt_nonce);
-                                                                $decryptor->adata_add('');
+                                                                $decryptor->adata_add("");
                                                                 my $data_len_pt = $decryptor->decrypt_add($data_len_ct);
                                                                 my $data_len_tag = substr($incoming_data,2,16);
                                                                 my $length_result = $decryptor->decrypt_done($data_len_tag);
@@ -375,7 +380,7 @@ package Net::Shadowsocks::Client;
                                                                         $decrypt_nonce = $decrypt_nonce->increment();
                                                                         $decryptor->reset();
                                                                         $decryptor->iv_add($decrypt_nonce);
-                                                                        $decryptor->adata_add('');
+                                                                        $decryptor->adata_add("");
                                                                         $decrypted_data .= $decryptor->decrypt_add(substr($incoming_data,18,$data_len ));
                                                                         my $data_tag = substr($incoming_data,18 + $data_len,16);
                                                                         my $data_result = $decryptor->decrypt_done($data_tag);
@@ -387,10 +392,12 @@ package Net::Shadowsocks::Client;
                                                                         else
                                                                         {
                                                                             $decrypt_nonce = $decrypt_nonce->increment();  
-                                                                            $decryptor->reset();     
+                                                                            #$decryptor->reset();    
+                                                                            #$decryptor->iv_add($decrypt_nonce);
+                                                                            #$decryptor->adata_add(""); 
                                                                             $incoming_data = substr($incoming_data,$chunk_len);                                                                       
                                                                             $rhandle->{rbuf} = $incoming_data;     
-                                                                            carp "more chunk to process" if (length($incoming_data ) > 0);                                                                                                      
+                                                                            #carp "more chunk to process" if (length($incoming_data ) > 0);                                                                                                      
                                                                         }  
                                                                     }      
                                                                 }
@@ -469,11 +476,11 @@ package Net::Shadowsocks::Client;
                                     else
                                     {
                                         my $header_len_pt = pack('n',length($plain_data));
-                                        my $header_len_ct_withtag = $encryptor ->ietf_encrypt($header_len_pt,'',$encrypt_nonce,$encrypt_subkey);
+                                        my $header_len_ct_withtag = $encryptor ->ietf_encrypt($header_len_pt,"",$encrypt_nonce,$encrypt_subkey);
                                         $encrypt_nonce = $encrypt_nonce->increment();
                                         #carp $encrypt_nonce;
                                         #carp ascii_to_hex($encrypt_nonce);
-                                        my $header_ct_withtag  =  $encryptor->ietf_encrypt($plain_data,'',$encrypt_nonce,$encrypt_subkey);
+                                        my $header_ct_withtag  =  $encryptor->ietf_encrypt($plain_data,"",$encrypt_nonce,$encrypt_subkey);
                                         $encrypt_nonce = $encrypt_nonce->increment();
                                         #carp $encrypt_nonce;
                                         #carp ascii_to_hex($encrypt_nonce);
@@ -483,25 +490,26 @@ package Net::Shadowsocks::Client;
                                 }
                                 elsif($self->{method} =~ /gcm$/)
                                 {
-                                     carp $encrypt_nonce;
+                                    #carp $encrypt_nonce;
+                                    $encryptor->reset();
                                     $encryptor->iv_add($encrypt_nonce);
-                                    carp length($plain_data);
+                                    #carp length($plain_data);
                                     my $header_len_pt = pack('n',length($plain_data));
-                                    carp $header_len_pt;
-                                    carp length($header_len_pt);
-                                    $encryptor->adata_add('');
+                                    #carp $header_len_pt;
+                                    #carp length($header_len_pt);
+                                    $encryptor->adata_add("");
                                     my $header_len_ct = $encryptor ->encrypt_add($header_len_pt,);
                                     my $header_len_tag = $encryptor->encrypt_done();
                                     $encrypt_nonce = $encrypt_nonce->increment();
                                     $encryptor->reset();
                                     $encryptor->iv_add($encrypt_nonce);
-                                    $encryptor->adata_add('');
+                                    $encryptor->adata_add("");
                                     my $header_ct = $encryptor->encrypt_add($plain_data);
                                     my $header_tag = $encryptor->encrypt_done();
                                     $encrypt_nonce = $encrypt_nonce->increment();
-                                    $encryptor->reset();
+                                    #$encryptor->reset();
                                     $encrypted_data = $header_len_ct . $header_len_tag . $header_ct . $header_tag;
-                                    carp length($encrypted_data);
+                                    #carp length($encrypted_data);
                                 }
 
                                 else
@@ -559,7 +567,7 @@ Net::Shadowsocks::Client - Shadowsocks protocol client module.
 
 =head1 VERSION
 
-Version 0.9.3
+Version 0.9.3.1
 
 =head1 SYNOPSIS
 
