@@ -31,7 +31,7 @@
     use Socket qw(IPPROTO_TCP TCP_FASTOPEN SOL_SOCKET SO_REUSEPORT SO_REUSEADDR);
     #use String::HexConvert ':all';
     
-    our $VERSION = '0.9.3';
+    our $VERSION = '0.9.3.1';
     
     $AnyEvent::Log::FILTER->level ("info");
     
@@ -100,7 +100,7 @@
                         else 
                         {
                             my $incoming_data = $client_buffer;
-                            my $decrypted_data = undef;
+                            my $decrypted_data = "";
                             unless ( defined($client_iv) ) 
                             {
                                  if(length($incoming_data) < length($iv) )
@@ -189,7 +189,7 @@
                                                             my $data_len_pt;
                                                             eval
                                                             {
-                                                                $data_len_pt = $decryptor->ietf_decrypt($data_len_ct, '', $decrypt_nonce, $decrypt_subkey);
+                                                                $data_len_pt = $decryptor->ietf_decrypt($data_len_ct, "", $decrypt_nonce, $decrypt_subkey);
                                                             };
                                                             if ( $@ ) 
                                                             {
@@ -208,7 +208,7 @@
                                                                 else
                                                                 {
                                                                     $decrypt_nonce = $decrypt_nonce->increment();
-                                                                    $decrypted_data .= $decryptor->ietf_decrypt(substr($incoming_data,18,$data_len + 16),'',$decrypt_nonce,$decrypt_subkey);
+                                                                    $decrypted_data .= $decryptor->ietf_decrypt(substr($incoming_data,18,$data_len + 16),"",$decrypt_nonce,$decrypt_subkey);
                                                                      if ( $@ ) 
                                                                      {
                                                                          AE::log error =>  "data forged!";
@@ -240,7 +240,7 @@
                                                             my $header_len_ct = substr($incoming_data,0,2); 
                                                             $decryptor->reset();
                                                             $decryptor->iv_add($decrypt_nonce);
-                                                            $decryptor->adata_add('');
+                                                            $decryptor->adata_add("");
                                                             my $header_len_pt = $decryptor->decrypt_add($header_len_ct);
                                                             my $header_len_tag = substr($incoming_data,2,16);
                                                             my $length_result = $decryptor->decrypt_done($header_len_tag);
@@ -263,7 +263,7 @@
                                                                     $decrypt_nonce = $decrypt_nonce->increment();
                                                                     $decryptor->reset();
                                                                     $decryptor->iv_add($decrypt_nonce);
-                                                                    $decryptor->adata_add('');
+                                                                    $decryptor->adata_add("");
                                                                     $decrypted_data .= $decryptor->decrypt_add(substr($incoming_data,18,$header_len ));
                                                                     my $header_tag = substr($incoming_data,18 + $header_len,16);
                                                                     my $header_result = $decryptor->decrypt_done($header_tag);
@@ -275,7 +275,7 @@
                                                                      else
                                                                      {
                                                                          $decrypt_nonce = $decrypt_nonce->increment();  
-                                                                         $decryptor->reset();
+                                                                         #$decryptor->reset();
                                                                          $incoming_data = substr($incoming_data,$chunk_len);                                                                       
                                                                          $chandle->{rbuf} = $incoming_data;   
                                                                          carp "more chunk to process" if (length($incoming_data ) > 0);                                                                      
@@ -436,35 +436,36 @@
                                                           else
                                                           {
                                                               my $data_len_pt = pack("n",length($plain_data));
-                                                              my $data_len_ct_withtag = $encryptor ->ietf_encrypt($data_len_pt,undef,$encrypt_nonce,$encrypt_subkey);
+                                                              my $data_len_ct_withtag = $encryptor ->ietf_encrypt($data_len_pt,"",$encrypt_nonce,$encrypt_subkey);
                                                               $encrypt_nonce = $encrypt_nonce->increment();
-                                                              my $data_ct_withtag  =  $encryptor->ietf_encrypt($plain_data,undef,$encrypt_nonce,$encrypt_subkey);
+                                                              my $data_ct_withtag  =  $encryptor->ietf_encrypt($plain_data,"",$encrypt_nonce,$encrypt_subkey);
                                                               $encrypt_nonce = $encrypt_nonce->increment();
                                                               $encrypted_data = $data_len_ct_withtag . $data_ct_withtag;
-                                                              carp length($encrypted_data);
+                                                              #carp length($encrypted_data);
                                                           }
                                                       }
                                                       elsif($self->{method} =~ /gcm$/)
                                                       {
-                                                          carp $encrypt_nonce;
+                                                          $encryptor->reset();
+                                                          #carp $encrypt_nonce;
                                                           $encryptor->iv_add($encrypt_nonce);
-                                                          carp length($plain_data);
+                                                          #carp length($plain_data);
                                                           my $header_len_pt = pack("n",length($plain_data));
-                                                          carp $header_len_pt;
-                                                          carp length($header_len_pt);
-                                                          $encryptor->adata_add(undef);
+                                                          #carp $header_len_pt;
+                                                          #carp length($header_len_pt);
+                                                          $encryptor->adata_add("");
                                                           my $header_len_ct = $encryptor ->encrypt_add($header_len_pt,);
                                                           my $header_len_tag = $encryptor->encrypt_done();
                                                           $encrypt_nonce = $encrypt_nonce->increment();
                                                           $encryptor->reset();
                                                           $encryptor->iv_add($encrypt_nonce);
-                                                          $encryptor->adata_add(undef);
+                                                          $encryptor->adata_add("");
                                                           my $header_ct = $encryptor->encrypt_add($plain_data);
                                                           my $header_tag = $encryptor->encrypt_done();
                                                           $encrypt_nonce = $encrypt_nonce->increment();
-                                                          $encryptor->reset();
+                                                          #$encryptor->reset();
                                                           $encrypted_data = $header_len_ct . $header_len_tag . $header_ct . $header_tag;
-                                                          carp length($encrypted_data);
+                                                          #carp length($encrypted_data);
                                                       }
                                                       else 
                                                       {
@@ -529,7 +530,7 @@
                                                             my $data_len_pt;
                                                             eval
                                                             {
-                                                                $data_len_pt = $decryptor->ietf_decrypt($data_len_ct, undef, $decrypt_nonce, $decrypt_subkey);
+                                                                $data_len_pt = $decryptor->ietf_decrypt($data_len_ct, "", $decrypt_nonce, $decrypt_subkey);
                                                             };
                                                             if ( $@ ) 
                                                             {
@@ -548,7 +549,7 @@
                                                                 else
                                                                 {
                                                                     $decrypt_nonce = $decrypt_nonce->increment();
-                                                                    $decrypted_data .= $decryptor->ietf_decrypt(substr($incoming_data,18,$data_len + 16),undef,$decrypt_nonce,$decrypt_subkey);
+                                                                    $decrypted_data .= $decryptor->ietf_decrypt(substr($incoming_data,18,$data_len + 16),"",$decrypt_nonce,$decrypt_subkey);
                                                                      if ( $@ ) 
                                                                      {
                                                                          AE::log error =>  "data forged!";
@@ -559,7 +560,7 @@
                                                                          $decrypt_nonce = $decrypt_nonce->increment();          
                                                                          $incoming_data = substr($incoming_data,$chunk_len);                                                                       
                                                                          $chandle->{rbuf} = $incoming_data;   
-                                                                         carp "more chunk to process" if (length($incoming_data ) > 0);                                                                                                            
+                                                                         #carp "more chunk to process" if (length($incoming_data ) > 0);                                                                                                            
                                                                      }  
                                                                 }      
                                                             }
@@ -578,8 +579,9 @@
                                                else
                                                {
                                                             my $header_len_ct = substr($incoming_data,0,2); 
+                                                            $decryptor->reset(); 
                                                             $decryptor->iv_add($decrypt_nonce);
-                                                            $decryptor->adata_add(undef);
+                                                            $decryptor->adata_add("");
                                                             my $header_len_pt = $decryptor->decrypt_add($header_len_ct);
                                                             my $header_len_tag = substr($incoming_data,2,16);
                                                             my $length_result = $decryptor->decrypt_done($header_len_tag);
@@ -602,7 +604,7 @@
                                                                     $decrypt_nonce = $decrypt_nonce->increment();
                                                                     $decryptor->reset();
                                                                     $decryptor->iv_add($decrypt_nonce);
-                                                                    $decryptor->adata_add(undef);
+                                                                    $decryptor->adata_add("");
                                                                     $decrypted_data = $decryptor->decrypt_add(substr($incoming_data,18,$header_len ));
                                                                     my $header_tag = substr($incoming_data,18 + $header_len,16);
                                                                     my $header_result = $decryptor->decrypt_done($header_tag);
@@ -614,10 +616,10 @@
                                                                      else
                                                                      {
                                                                          $decrypt_nonce = $decrypt_nonce->increment();  
-                                                                         $decryptor->reset();      
+                                                                         #$decryptor->reset();      
                                                                          $incoming_data = substr($incoming_data,$chunk_len);                                                                       
                                                                          $chandle->{rbuf} = $incoming_data;     
-                                                                          carp "more chunk to process" if (length($incoming_data ) > 0);                                                                
+                                                                         #carp "more chunk to process" if (length($incoming_data ) > 0);                                                                
                                                                      }  
                                                                 }      
                                                             }
@@ -697,7 +699,7 @@ Net::Shadowsocks::Server - Shadowsocks protocol server module.
 
 =head1 VERSION
 
-Version 0.9.3
+Version 0.9.3.1
 
 =head1 SYNOPSIS
 
